#ifndef _LINUX_SWSUSP_H
#define _LINUX_SWSUSP_H

#ifdef CONFIG_X86
#include <asm/suspend.h>
#endif

#include <linux/kernel.h>
extern char __nosave_begin, __nosave_end;

#ifdef CONFIG_PM

#include <linux/init.h>

/* For swsusp */
#include <linux/swap.h>

#define SUSPEND_CORE_VERSION "2.1.5"
#ifndef KERNEL_POWER_SWSUSP_C
#define name_suspend "Software Suspend " SUSPEND_CORE_VERSION ": "
#endif

extern unsigned long suspend_action;
extern unsigned long suspend_result;
extern unsigned long suspend_debug_state;

#define TEST_RESULT_STATE(bit) (test_bit(bit, &suspend_result))
#define SET_RESULT_STATE(bit) (test_and_set_bit(bit, &suspend_result))
#define CLEAR_RESULT_STATE(bit) (test_and_clear_bit(bit, &suspend_result))

#define TEST_ACTION_STATE(bit) (test_bit(bit, &suspend_action))
#define SET_ACTION_STATE(bit) (test_and_set_bit(bit, &suspend_action))
#define CLEAR_ACTION_STATE(bit) (test_and_clear_bit(bit, &suspend_action))

#ifdef CONFIG_SOFTWARE_SUSPEND_DEBUG
#define TEST_DEBUG_STATE(bit) (test_bit(bit, &suspend_debug_state))
#define SET_DEBUG_STATE(bit) (test_and_set_bit(bit, &suspend_debug_state))
#define CLEAR_DEBUG_STATE(bit) (test_and_clear_bit(bit, &suspend_debug_state))
#else
#define TEST_DEBUG_STATE(bit) (0)
#define SET_DEBUG_STATE(bit) (0)
#define CLEAR_DEBUG_STATE(bit) (0)
#endif

/* first status register - this is suspend's return code. */
#define SUSPEND_ABORTED			0
#define SUSPEND_ABORT_REQUESTED		1
#define SUSPEND_NOSTORAGE_AVAILABLE	2
#define SUSPEND_INSUFFICIENT_STORAGE	3
#define SUSPEND_FREEZING_FAILED		4
#define SUSPEND_UNEXPECTED_ALLOC	5
#define SUSPEND_KEPT_IMAGE		6
#define SUSPEND_WOULD_EAT_MEMORY	7
#define SUSPEND_UNABLE_TO_FREE_ENOUGH_MEMORY 8

/* second status register */
#define SUSPEND_REBOOT		0
#define SUSPEND_PAUSE		2
#define SUSPEND_SLOW		3
#define SUSPEND_NOPAGESET2	7
#define SUSPEND_LOGALL		8
/* Set to disable compression when compiled in */
#define SUSPEND_NO_COMPRESSION	9
//#define SUSPEND_ENABLE_KDB	10
#define SUSPEND_CAN_CANCEL	11
#define SUSPEND_KEEP_IMAGE	13
#define SUSPEND_FREEZER_TEST	14
#define SUSPEND_FREEZER_TEST_SHOWALL 15
#define SUSPEND_SINGLESTEP	16
#define SUSPEND_PAUSE_NEAR_PAGESET_END 17
#define SUSPEND_USE_ACPI_S4	18
#define SUSPEND_KEEP_METADATA	19
#define SUSPEND_TEST_FILTER_SPEED	20

/* debug sections  - if debugging compiled in */
#define SUSPEND_ANY_SECTION	0
#define SUSPEND_FREEZER		1
#define SUSPEND_EAT_MEMORY 	2
#define SUSPEND_PAGESETS	3
#define SUSPEND_IO		4
#define SUSPEND_BMAP		5
#define SUSPEND_HEADER		6
#define SUSPEND_WRITER		9
#define SUSPEND_MEMORY		10
#define SUSPEND_RANGES		11
#define SUSPEND_SPINLOCKS	12
#define SUSPEND_MEM_POOL	13
#define SUSPEND_RANGE_PARANOIA	14
#define SUSPEND_NOSAVE		15
#define SUSPEND_INTEGRITY	16
/* debugging levels. */
#define SUSPEND_STATUS		0
#define SUSPEND_ERROR		2
#define SUSPEND_LOW	 	3
#define SUSPEND_MEDIUM	 	4
#define SUSPEND_HIGH	  	5
#define SUSPEND_VERBOSE		6

extern void __suspend_message(unsigned long section, unsigned long level, int log_normally,
		const char *fmt, ...);

#ifdef CONFIG_SOFTWARE_SUSPEND_DEBUG
extern int suspend_memory_pool_level(int only_lowmem);
#define suspend_message(sn, lev, log, fmt, a...) \
do { \
	if (TEST_DEBUG_STATE(sn)) \
		suspend2_core_ops->suspend_message(sn, lev, log, fmt, ##a); \
} while(0)
#define PRINTFREEMEM(desn) \
	suspend_message(SUSPEND_MEMORY, SUSPEND_MEDIUM, 1, \
		"Free memory %s: %d+%d.\n", desn, \
		nr_free_pages() + suspend_amount_grabbed, \
		suspend_memory_pool_level(0));
#else /* CONFIG_SOFTWARE_SUSPEND_DEBUG */
#define PRINTFREEMEM(desn) do { } while(0)
#define suspend_message(sn, lev, log, fmt, a...) \
do { \
	if (lev == 0) \
		suspend2_core_ops->suspend_message(sn, lev, log, fmt, ##a); \
} while(0)
#endif /* CONFIG_SOFTWARE_SUSPEND_DEBUG */
  
#ifdef CONFIG_SMP
extern void disable_nonboot_cpus(void);
extern void enable_nonboot_cpus(void);
#else
static inline void disable_nonboot_cpus(void) {}
static inline void enable_nonboot_cpus(void) {}
#endif

extern int software_suspend(void);
	
/* Suspend 2 */

#define SUSPEND_DISABLED		0
#define SUSPEND_RUNNING			1
#define SUSPEND_RESUME_DEVICE_OK	2
#define SUSPEND_NORESUME_SPECIFIED	3
#define SUSPEND_COMMANDLINE_ERROR 	4
#define SUSPEND_IGNORE_IMAGE		5
#define SUSPEND_SANITY_CHECK_PROMPT	6
#define SUSPEND_FREEZER_ON		7
#define SUSPEND_DISABLE_SYNCING		8
#define SUSPEND_BLOCK_PAGE_ALLOCATIONS	9
#define SUSPEND_USE_MEMORY_POOL		10
#define SUSPEND_STAGE2_CONTINUE		11
#define SUSPEND_FREEZE_SMP		12
#define SUSPEND_PAGESET2_NOT_LOADED	13
#define SUSPEND_CONTINUE_REQ		14
#define SUSPEND_RESUMED_BEFORE		15
#define SUSPEND_RUNNING_INITRD		16
#define SUSPEND_RESUME_NOT_DONE		17
#define SUSPEND_BOOT_TIME		18
#define SUSPEND_NOW_RESUMING		19
#define SUSPEND_SLAB_ALLOC_FALLBACK	20
#define SUSPEND_IGNORE_LOGLEVEL		21

extern unsigned long software_suspend_state;
#define test_suspend_state(bit) \
	(test_bit(bit, &software_suspend_state))

#define clear_suspend_state(bit) \
	(clear_bit(bit, &software_suspend_state))

#define set_suspend_state(bit) \
	(set_bit(bit, &software_suspend_state))

#define get_suspend_state() 		(software_suspend_state)
#define restore_suspend_state(saved_state) \
	do { software_suspend_state = saved_state; } while(0)
	
/* kernel/suspend.c */
extern void suspend_try_suspend(void);
extern unsigned int suspend_task;

/* Kernel threads are type 3 */
#define FREEZER_ALL_THREADS 0
#define FREEZER_KERNEL_THREADS 3

extern int freeze_processes(int no_progress);
extern void thaw_processes(int which_threads);

extern int pm_prepare_console(void);
extern void pm_restore_console(void);

#define SUSPEND_KEY_KEYBOARD 1
#define SUSPEND_KEY_SERIAL 2

struct page;

struct suspend2_core_ops {
	/* Entry points for suspending & resuming */
	void (* do_suspend) (void);
	int (* do_resume) (void);

	/* Pre and post lowlevel routines */
	void (* suspend1) (void);
	void (* suspend2) (void);
	void (* resume1) (void);
	void (* resume2) (void);
	
	void (* free_pool_pages) (struct page *page, unsigned int order);
	struct page * (* get_pool_pages) (unsigned int gfp_mask, unsigned int order);

	unsigned long (* get_grabbed_pages) (int order);
	void (* cleanup_finished_io) (void);

	void (* suspend_message) (unsigned long, unsigned long, int, const char *, ...);
	unsigned long (* update_status) (unsigned long value, unsigned long maximum,
		const char *fmt, ...);
	void (*prepare_status) (int printalways, int clearbar, const char *fmt, ...);
	void (* schedule_message) (int message_number);
	void (* early_boot_plugins) (void);
	int (* keypress) (unsigned int keycode);

	void (* verify_checksums) (void);
};
extern volatile struct suspend2_core_ops * suspend2_core_ops;
#ifdef CONFIG_SOFTWARE_SUSPEND2
extern void software_suspend_try_resume(void);
extern void suspend_handle_keypress(unsigned int keycode, int source);
#else
#define software_suspend_try_resume()	do { } while(0)
#define suspend_handle_keypress(a, b) do { } while(0)
#endif

#define suspend2_free_pool_pages(page, order)	suspend2_core_ops->free_pool_pages(page, order)
#define suspend2_get_pool_pages(mask, order)	suspend2_core_ops->get_pool_pages(mask, order)
#define suspend2_get_grabbed_pages(order)	suspend2_core_ops->get_grabbed_pages(order)
#define suspend2_cleanup_finished_io()		suspend2_core_ops->cleanup_finished_io()
#define suspend2_verify_checksums()		suspend2_core_ops->verify_checksums()

#else /* CONFIG_PM off */

#define suspend_try_suspend()		do { } while(0)
#define suspend_task			(0)
#define software_suspend_state		(0)
#define test_suspend_state(bit) 	(0)
#define clear_suspend_state(bit)	do { } while (0)
#define set_suspend_state(bit)		do { } while(0)
#define get_suspend_state() 		(0)
#define restore_suspend_state(saved_state) do { } while(0)
#define software_suspend_try_resume()	do { } while(0)

static inline int suspend_bug(void)
{
	BUG();
	return 0;
}

#define suspend2_free_pool_pages(page, order) suspend_bug()
#define suspend2_get_pool_pages(mask, order) (struct page *) suspend_bug()
#define suspend2_get_grabbed_pages(order) (struct page *) suspend_bug()
#define suspend2_cleanup_finished_io()	do { BUG(); } while(0)
#define suspend2_verify_checksums() do { BUG(); } while(0)

static inline int software_suspend(void)
{
	printk("Warning: fake suspend called\n");
	return -EPERM;
}
#define software_resume()		do { } while(0)
#define suspend_handle_keypress(a, b) do { } while(0)
#endif

#endif /* _LINUX_SWSUSP_H */
